

"""
Evalulate COSMO-GHG clouds with DWD observations in Berlin

Berlin stations:
    - Berlin-Dahlem (id: 403)
    - Berlin-Schonefeld (id: 427)
    - Berlin-Tegel (id: 430)

data downloaded here:
    ftp://ftp-cdc.dwd.de/pub/CDC/observations_germany/climate/hourly/cloudiness/historical/
"""

from datetime import datetime, timedelta

ID2LOCATION = {
    403: (13.3017, 52.4537),
    427: (13.5306, 52.3807),
    430: (13.3088, 52.5644), # lon, lat
}


"""
Evalulate COSMO-GHG clouds with MODIS cloud mask
"""

import glob

import netCDF4
import numpy as np
import pandas
import scipy.interpolate


import amrs.misc
from . import misc
from . import config as cfg
from . import io


def download_modis_cloud_mask():
    pass


def read_cloud_mask(filename):
    """
    Returns masked arrary where MODIS pixels are cloudy (00) or
    probably cloudy (00 and 01)

    MODIS cloud mask is a bit array with bit fields 1 and 2 decode
    the FOV quality flag with:

    - 00: cloudy
    - 01: uncertain clear (probably cloudy)
    - 10: probably clear
    - 11: clear
    """
    with netCDF4.Dataset(filename) as nc:
        mask = nc.variables['Cloud_Mask'][0].data

    # check if 1st and 2nd bits are 1 and 1
    first = (mask & 2**2 != 0)

    return first == False # cloudy and probably cloudy

    second = (mask & 2**1 != 0)
    return (first == False) & (second == False) # cloudy


def get_lonlat(filename):
    """
    Returns longitude/latitude at cloud mask at 1km resolution.

    The MODIS file stores only 5km lon/lat which needs to be interpolated to
    the 1km resolution.
    """
    with netCDF4.Dataset(filename) as nc:

        time = nc.variables['Scan_Start_Time'][:].data.mean()

        lon = nc.variables['Longitude'][:].data
        lat = nc.variables['Latitude'][:].data

        nx, ny = nc.variables['Cloud_Mask'].shape[1:]

    # time
    time = datetime(1993,1,1) + timedelta(seconds=time)
    time = pandas.Timestamp(time)

    # lon/lat are center coords on a 5x5 km grid
    x = np.linspace(0,1,nx)
    y = np.linspace(0,1,ny)

    X, Y = np.meshgrid(x,y)


    # coords of lon and lat at low resolution
    xg = x[2::5][:lon.shape[0]]
    yg = y[2::5][:lat.shape[1]]

    # interpolate lon/lat to high resolution
    ifunc = scipy.interpolate.RectBivariateSpline(xg,yg,lon)
    lon = ifunc(x,y)

    ifunc = scipy.interpolate.RectBivariateSpline(xg,yg,lat)
    lat = ifunc(x,y)

    return time, lon, lat



def read_modis(filename):
    # project is_cloudy flag to cosmo grid

    # load cloud mask
    is_cloudy = read_cloud_mask(filename)
    time, lon, lat = get_lonlat(filename)

    # convert to rotated pole coords
    rlon, rlat = misc.rotpole2wgs(lon, lat, inverse=True)

    # corners of COSMO grid
    rlonc, rlatc = cfg.rlonc, cfg.rlatc


    # 
    valids = np.isfinite(is_cloudy)


    # bin to COSMO grid
    H, _, _ = np.histogram2d(rlat[valids], rlon[valids], bins=[rlatc, rlonc],
                             weights=is_cloudy[valids])

    N, _, _ = np.histogram2d(rlat[valids], rlon[valids], bins=[rlatc, rlonc])

    is_cloudy = H / N


    # fill small gaps
    #is_cloudy = amrs.misc.normalized_convolution(is_cloudy, np.ones((3,3)))
    #is_cloudy = is_cloudy > 0.5

    return time, np.ma.array(is_cloudy, dtype='bool', mask=np.isnan(is_cloudy))



def cloud_fraction(is_cloudy):

    if hasattr(is_cloudy, 'mask'):
        valids = ~is_cloudy.mask
    else:
        valids = Ellipsis

    cloudy = np.sum(is_cloudy[valids] == True)
    clear = np.sum(is_cloudy[valids] == False)

    fraction = clear / (cloudy + clear)

    return clear, cloudy, fraction



def read_modis_day(filenames, reduce=True):

    times = []
    modis_mask = []
    valids = []

    for filename in filenames:

        time, mask = read_modis(filename)

        if mask.mask.all(): # all cells are masked
            continue

        if not (8 <= time.round('H').hour <= 12):
            print('Reject time: %s' % time)
            continue

        valids.append(np.sum(mask.mask == False))
        times.append(time)
        modis_mask.append(mask)

    modis_mask = np.ma.array(modis_mask)

    if reduce:
        modis_mask = modis_mask.mean(axis=0) > 0.5

        mean_time = 0.0
        for i,t in enumerate(times):
            t = (t - datetime(1993,1,1)).total_seconds()
            mean_time += t * valids[i] / sum(valids)

        mean_time = datetime(1993,1,1) + timedelta(seconds=mean_time)
        mean_time = pandas.Timestamp(mean_time)

        return mean_time, modis_mask

    return np.array(times), modis_mask


def main(start=1, stop=359):

    times = []
    cloud_info = []

    for day in range(start, stop):

        filenames = sorted(glob.glob('%03d/*.hdf' % day))

        if not filenames:
            print('No filenames for day %d' % day)
            continue

        # read MODIS data
        time, modis_mask = read_modis_day(filenames)

        starttime = {
            True: datetime(2015,7,1),
            False: datetime(2015,1,1)
        }[time >= datetime(2015,7,1)]

        # read COSMO cloud mask
        clct = io.read_field(time.round('H'), starttime, keys=['CLCT'],
                             input_files='2D', path='/output/SMARTCARB/output/')

        cosmo_mask = clct[0,:,:] > 0.01
        cosmo_mask = np.ma.array(cosmo_mask, mask=modis_mask.mask)


        modis_clear, modis_cloudy, modis_fraction = cloud_fraction(modis_mask)
        cosmo_clear, cosmo_cloudy, cosmo_fraction = cloud_fraction(cosmo_mask)


        print(time.round('H'), '%.3f' % modis_fraction, '%.3f' % cosmo_fraction)

        times.append(time)
        cloud_info.append([modis_clear, modis_cloudy, modis_fraction,
                           cosmo_clear, cosmo_cloudy, cosmo_fraction])

    columns = ['MODIS clear', 'MODIS cloudy', 'MODIS clear fraction',
               'COSMO clear', 'COSMO cloudy', 'COSMO clear fraction']

    data = pandas.DataFrame(cloud_info, index=times, columns=columns)
    data['valids'] = data['MODIS clear'] + data['MODIS cloudy']


    return data














